@@ -114,28 +114,28 @@ Xfm*file window*button box.?.borderWidth: 0
 ! for *both* application and file windows.
 
 ! The background color for file and application windows
-Xfm*background: lightgray
+Xfm*background: grey77
 
 ! The foreground color for icon toggles; this is also used when an icon toggle
 ! is highlighted or marked
 #ifdef ENHANCE_SELECTION
-Xfm.highlightColor: red
+Xfm.highlightColor: midnight blue
 #else
-Xfm*viewport.icon box*Toggle.foreground: red
+Xfm*viewport.icon box*Toggle.foreground: midnight blue
 #ifdef USE_NEW_WIDGETS
-Xfm*TextFileList.highlightColor: red
-Xfm*IconFileList.highlightColor: red
+Xfm*TextFileList.highlightColor: midnight blue
+Xfm*IconFileList.highlightColor: midnight blue
 #endif
 #endif
 
 #ifdef ENHANCE_LOG
 ! Some nice settings for the log window
 xfm*fmLog*background:			white
-xfm*fmLog.form.background:		lightgray
-xfm*fmLog*Command*background:		lightgray
-xfm*fmLog*Label*background:		lightgray
-xfm*fmLog*Toggle*background:		lightgray
-xfm*fmLog*Scrollbar*background:		lightgray
+xfm*fmLog.form.background:		grey77
+xfm*fmLog*Command*background:		grey77
+xfm*fmLog*Label*background:		grey77
+xfm*fmLog*Toggle*background:		grey77
+xfm*fmLog*Scrollbar*background:		grey77
 #endif
 
 ! Use these only if you  have the 75dpi fonts. (recommended)
@@ -295,6 +295,10 @@ Xfm*file window*folderlabel.borderWidth: 0
 Xfm*icon box*shadowWidth: 0
 Xfm*MenuButton.highlightThickness: 0
 Xfm*Command.highlightThickness: 0
+Xfm*Toggle.highlightThickness: 0
+Xfm*Label.ShadowWidth: 0
+Xfm*Label*borderWidth: 0
+Xfm*status*shadowWidth: 2
 #endif
 #ifdef ENHANCE_TRANSLATIONS
 Xfm*file window*icon box*tree_icon.baseTranslations: #override\n\
@@ -465,6 +469,11 @@ Xfm*file window*icon box.baseTranslations: #override\n\
 #endif
 #ifdef ENHANCE_POP_ACCEL
 Xfm*button box*cancel.accelerators:#override\n\
+<ClientMessage>WM_PROTOCOLS: set() notify() unset()\n\
+<Key>Escape: set() notify() unset()\n
+
+Xfm*about*button box*ok.accelerators:#override\n\
+<ClientMessage>WM_PROTOCOLS: set() notify() unset()\n\
 <Key>Escape: set() notify() unset()\n
 
 Xfm*button box*ok.accelerators:#override\n\
@@ -480,6 +489,7 @@ Xfm*button box*install.accelerators:#override\n\
 <Key>Linefeed: set() notify() unset()\n
 #ifdef ENHANCE_LOG
 Xfm*fmLog*Hide.accelerators:#override\n\
+<ClientMessage>WM_PROTOCOLS: set() notify() unset()\n\
 <Key>Return: set() notify() unset()\n\
 <Key>Linefeed: set() notify() unset()\n
 Xfm*fmLog*Clear Log.accelerators:#override\n\
