@@ -39,6 +39,11 @@
 #endif
 #endif
 
+/*  RBW - 2001/08/14  */
+#ifdef _SVID_SOURCE
+#define SYSV
+#endif
+
 #ifdef SYSV
 #define getwd(s) getcwd(s,MAXPATHLEN)
 #endif
@@ -125,6 +130,7 @@ typedef struct _FileWindowRec {
   int n_files;
   int n_selections;
   long n_bytes, n_bytes_selected;
+  int	option_in_use;
   Widget *file_items, *folder_items, *view_items, *option_items;
   Widget unreadable;
   Boolean do_filter;             /* KMR */
@@ -542,10 +548,12 @@ Widget *createFloatingMenu(String menu_name,
 char *varPopup(Pixmap icon_bm, char *action);
 void aboutPopup(void);
 
+void setWMProps(Widget shell);
+
 /*--FmComms------------------------------------------------------------------*/
 
 extern Atom xfm_open_window, xfm_update_window, wm_delete_window,
-    wm_protocols;
+    wm_protocols, wm_save_yourself, kwm_save_yourself;
 void clientMessageHandler(Widget w, XtPointer closure, XEvent *e);
 void initComms(void);
 
