@@ -155,6 +155,7 @@ static char iconbox_translations[] = "\
 /*-----------------------------------------------------------------------------
   Action tables
 -----------------------------------------------------------------------------*/
+static void CatchEntryLeave2(Widget,XtPointer,XEvent*,Boolean *ctd);
 
 static void dummy(Widget w, XEvent *event, String *params, 
 		       Cardinal *num_params) {}
@@ -389,6 +390,17 @@ void createApplicationDisplay()
       XtVaGetValues(aw.apps[i].toggle, XtNbackground, &back, NULL);
       XtVaSetValues(aw.apps[i].toggle, XtNborder, (XtArgVal) back, NULL);
 
+/*
+    RBW Test stuff...
+*/
+XtInsertEventHandler(aw.apps[i].toggle,
+                     EnterWindowMask|LeaveWindowMask,
+                     False,
+                     (XtEventHandler)CatchEntryLeave2,
+                     (XtPointer)0,
+                     XtListHead);
+
+
       icon_label_args[1].value = (XtArgVal) aw.apps[i].toggle;
       icon_label_args[2].value = (XtArgVal) aw.apps[i].name;
       aw.apps[i].label = XtCreateManagedWidget("label", labelWidgetClass,
@@ -626,3 +638,26 @@ void clearApplicationsStack()
   n_appst = 0;
 }
 
+
+/*
+    RBW Test stuff...
+*/
+
+
+void CatchEntryLeave2(Widget w, XtPointer cld, XEvent* ev, Boolean *ctd)
+{
+/*
+    RBW - eat extraneous LeaveNotify/EnterNotify events caused by some window
+    managers when a ButtonPress happens, so that the Xt translation mechanism
+    for double-clicks doesn't get confused.
+*/
+if (ev->xcrossing.mode == NotifyGrab || ev->xcrossing.mode == NotifyUngrab)  {
+  *ctd = False;  /* Eat this event */
+  }  else  {
+  *ctd = True;  /* Don't eat this event */
+  }
+return;
+}
+
+
+
