@@ -316,18 +316,6 @@ static ButtonRec chmod_buttons[] = {
   { "cancel", "Cancel", chmodCancelCb }
 };
 
-/*---------------------------------------------------------------------------
-  WM_DELETE_WINDOW handler
----------------------------------------------------------------------------*/
-
-static void clmHandler(Widget w, XtPointer cld, XEvent *ev, Boolean *ctd)
-{
-  XClientMessageEvent *cme = (XClientMessageEvent *)ev;
-  if (ev->type != ClientMessage || cme->message_type != wm_protocols)
-    return;
-  chmodCancelCb(w, NULL, NULL);
-}
-
 /*---------------------------------------------------------------------------
   PUBLIC FUNCTIONS
 ---------------------------------------------------------------------------*/
@@ -396,13 +384,11 @@ void createChmodPopup()
 
 #ifdef ENHANCE_POP_ACCEL
   XtInstallAllAccelerators(form,form);
+  XtInstallAllAccelerators(chmode.shell,form);
 #endif
 
   XtRealizeWidget(chmode.shell);
-  XSetWMProtocols(XtDisplay(chmode.shell), XtWindow(chmode.shell),
-		  &wm_delete_window, 1);
-  XtAddEventHandler(chmode.shell, (EventMask)0L, True,
-		    (XtEventHandler)clmHandler, (XtPointer)NULL);
+  setWMProps(chmode.shell);
 }
 
 /*---------------------------------------------------------------------------*/
