@@ -14,5 +14,7 @@
 #define XFM_UPDATE_WINDOW "XFM_UPDATE_WINDOW"
 #define WM_DELETE_WINDOW  "WM_DELETE_WINDOW"
 #define WM_PROTOCOLS      "WM_PROTOCOLS"
+#define WM_SAVE_YOURSELF  "WM_SAVE_YOURSELF"
+#define KWM_SAVE_YOURSELF "KWM_SAVE_YOURSELF"
 
 #endif
