@@ -97,18 +97,6 @@ static ButtonRec confirm_buttons[] = {
 };
 
 
-/*---------------------------------------------------------------------------
-  WM_DELETE_WINDOW handler
----------------------------------------------------------------------------*/
-
-static void clmHandler(Widget w, XtPointer cld, XEvent *ev, Boolean *ctd)
-{
-  XClientMessageEvent *cme = (XClientMessageEvent *)ev;
-  if (ev->type != ClientMessage || cme->message_type != wm_protocols)
-    return;
-  confirmAbortCb(w, NULL, NULL);
-}
-
 /*-----------------------------------------------------------------------------
   PUBLIC FUNCTIONS
 -----------------------------------------------------------------------------*/
@@ -150,13 +138,11 @@ void createConfirmPopup()
 
 #ifdef ENHANCE_POP_ACCEL
   XtInstallAllAccelerators(form,form);
+  XtInstallAllAccelerators(shell,form);
 #endif
 
   XtRealizeWidget(shell);
-  XSetWMProtocols(XtDisplay(shell), XtWindow(shell),
-		  &wm_delete_window, 1);
-  XtAddEventHandler(shell, (EventMask)0L, True,
-		    (XtEventHandler)clmHandler, (XtPointer)NULL);
+  setWMProps(shell);
 }
 
 /*---------------------------------------------------------------------------*/
