@@ -35,6 +35,8 @@
 #include <X11/Xaw/Toggle.h>
 #include <X11/Xaw/Label.h>
 
+#include "XtHelper.h"
+
 #include "TextFileList.h"
 #include "IconFileList.h"
 
@@ -426,10 +428,26 @@ static int longestName(FileWindowRec *fw)
   return longest;
 }
 
-static void unmapTxtOptsCb(Widget w, XtPointer cld, XtPointer cad)
+static void
+unmapTxtOptsCb(Widget w, XtPointer cld, XtPointer cad)
 {
- Widget options_button=(Widget) cld;
- XtUnmapWidget(options_button);
+ FileWindowRec	*fw=(FileWindowRec *)cld;
+ Widget			options_button;
+
+ /* don't do nothing if our parent is already being
+  * destroyed.
+  * (fw may be a dangling pointer in this case!)
+  */
+ if (XtIsBeingDestroyed(XtParent(w))) return;
+
+ /* if they continue to use the options button, return */
+ if (--fw->option_in_use >0) return;
+
+ options_button=XtParent(XtParent(fw->option_items[0]));
+ if (XtIsRealized(options_button))
+   XtUnmapWidget(options_button);
+ else
+   XtSetMappedWhenManaged(options_button,False);
 }
 
 /* This callback is registered on the callback list of a fileList widget.
@@ -812,19 +830,21 @@ static void createTextDisplay(FileWindowRec *fw)
 
  XtAddCallback(fw->icon_box,XtNcallback,fileListCb,(XtPointer)fw);
 
- options_button=XtParent(XtParent(fw->option_items[0]));
-
- /* Note: we can not pass 'fw' to the procedure on the 'destroyCallback'
-  *       list, because *fw may have been freed already at the time
-  *	  the callback is invoked (see fileCloseCb);
+ /* increment in_use count for the options button; the unmapTxtOptsCb
+  * will only unmap the button if the count drops to zero.
   */
- XtAddCallback(fw->icon_box,XtNdestroyCallback,
-		unmapTxtOptsCb,(XtPointer)options_button);
+ fw->option_in_use++;
+ XtAddCallback(fw->icon_box,XtNdestroyCallback, unmapTxtOptsCb,(XtPointer)fw);
  
  updateTxtOpts(fw);
 
+ options_button=XtParent(XtParent(fw->option_items[0]));
+
  /* show the text options menu */
- XtMapWidget(options_button);
+ if (XtIsRealized(fw->shell))
+   XtMapWidget(options_button);
+ else
+   XtSetMappedWhenManaged(options_button,True);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1174,6 +1194,11 @@ static FileWindowRec *createFileWindow(String path, String title,
   fw->option_items=createMenu("options","Options",text_opts_menu,XtNumber(text_opts_menu),
 			      16, fw->button_box, (XtPointer) fw);
   XtSetMappedWhenManaged(XtParent(XtParent(fw->option_items[0])),False);
+  fw->option_in_use=0;
+  /* count the number of textFileLists currently
+   * using the options menu (maybe 2; one, old, being destroyed + a
+   * a new one.
+   */
 
   /* create folder label */
   label_args[0].value = (XtArgVal) fw->button_box;
@@ -1220,8 +1245,7 @@ void newFileWindow(String path, DisplayType d, Boolean by_cursor)
   XtRealizeWidget(fw->shell);
   XSetIconName(XtDisplay(fw->shell), XtWindow(fw->shell), fw->directory);
   XStoreName(XtDisplay(fw->shell), XtWindow(fw->shell), fw->directory);
-  XSetWMProtocols(XtDisplay(fw->shell), XtWindow(fw->shell),
-	  	  &wm_delete_window, 1);
+  setWMProps(fw->shell);
   XtAddEventHandler(fw->shell, (EventMask)0L, True,
 		    (XtEventHandler)clientMessageHandler, (XtPointer)NULL);
   if (by_cursor)
