@@ -4,6 +4,9 @@
   (c) Simon Marlow 1990-92
   (c) Albert Graef 1994
 
+  modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
+  different enhancements (see README-NEW).
+
   Action procedures for widgets in a file window
 ---------------------------------------------------------------------------*/
 
@@ -14,17 +17,22 @@
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Xaw/Toggle.h>
+#include "FileList.h"
 
 #include "Am.h"
 #include "Fm.h"
 
+#ifdef ENHANCE_SELECTION
+#include "FmSelection.h"
+#endif
+
 /*---------------------------------------------------------------------------
   PUBLIC DATA
 ---------------------------------------------------------------------------*/
 
 MoveInfo move_info;
 Boolean dragging = False;
-Cursor drag_cursor;
+Cursor drag_cursor, cache_cursor;
 
 /*---------------------------------------------------------------------------
   STATIC DATA
@@ -37,7 +45,7 @@ static int fromi, toi;
   PRIVATE FUNCTIONS
 ---------------------------------------------------------------------------*/
 
-static int findWidget(Widget w, FileWindowRec **fw_ret)
+static int findWidget(Widget w, FileWindowRec **fw_ret, XEvent *ev)
 {
   int i;
   FileWindowRec *fw;
@@ -45,6 +53,9 @@ static int findWidget(Widget w, FileWindowRec **fw_ret)
   for (fw = file_windows; fw; fw = fw->next) {
     if (fw->icon_box == w || fw->label == w) {
       *fw_ret = fw;
+      if (XtIsSubclass(w,fileListWidgetClass)) {
+	return FileListFindEntry((FileListWidget)w,ev) ;
+      }
       return -1;
     }
     for (i = 0; i < fw->n_files; i++)
@@ -92,7 +103,7 @@ static int moveFiles()
 	  else
 	    continue;
       }
-      if (rename(from,to)) {
+      if (rmove(from,to)) {
 	char s[0xff];
 	sprintf(s, "Error moving %s:", move_info.fw->files[i]->name);
 	sysError(s);
@@ -174,7 +185,11 @@ static int setupMoveCopy(FileWindowRec *fw, int i)
 
 static void fileEndMove(FileWindowRec *fw, int i)
 {
+  char fromdir[MAXPATHLEN], todir[MAXPATHLEN];
+
   if (!setupMoveCopy(fw, i)) return;
+  strcpy(fromdir, from);
+  strcpy(todir, to);
 
   if (!strcmp(from, to)) {
     error("Move:", "Source and destination are identical");
@@ -199,7 +214,7 @@ static void fileEndMove(FileWindowRec *fw, int i)
   }
 
   if (moveFiles()) {
-    markForUpdate(from); markForUpdate(to);
+    markForUpdate(fromdir); markForUpdate(todir);
     intUpdate();
   }
 
@@ -211,7 +226,10 @@ static void fileEndMove(FileWindowRec *fw, int i)
 
 static void fileEndCopy(FileWindowRec *fw, int i)
 {
+  char todir[MAXPATHLEN];
+
   if (!setupMoveCopy(fw, i)) return;
+  strcpy(todir, to);
 
   if (!strcmp(from, to)) {
     error("Copy:", "Source and destination are identical");
@@ -236,7 +254,7 @@ static void fileEndCopy(FileWindowRec *fw, int i)
   }
 
   if (copyFiles()) {
-    markForUpdate(to);
+    markForUpdate(todir);
     intUpdate();
   }
 
@@ -299,10 +317,9 @@ static void fileEndAction(FileWindowRec *fw, int i)
    button on */
 
 static Widget drag(Widget w, int button, FileWindowRec *fw,
-		   Boolean *on_root_return)
+		   Boolean *on_root_return, XEvent *event_return)
 {
   Cursor c;
-  XEvent e;
   int i, x, y;
   Window child;
 
@@ -333,7 +350,7 @@ static Widget drag(Widget w, int button, FileWindowRec *fw,
     move_info.type = MultipleFiles;
     c = curs[FILES_CUR];
   }
-  drag_cursor = c;
+  cache_cursor=drag_cursor = c;
 
   XGrabPointer(XtDisplay(w), XtWindow(w), True,
 	       EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
@@ -343,34 +360,40 @@ static Widget drag(Widget w, int button, FileWindowRec *fw,
   freeze = dragging = True;
   
   for (;;) {
-    XtAppNextEvent(app_context, &e);
-    switch (e.type) {
+    XtAppNextEvent(app_context, event_return);
+    switch (event_return->type) {
     case ButtonPress:        /* Ignore button presses */
       continue;
     case ButtonRelease:      /* Ignore button releases except 2 */
-      if (e.xbutton.button != button)
+      if (event_return->xbutton.button != button)
 	continue;
       break;
     default:
-      XtDispatchEvent(&e);
+      XtDispatchEvent(event_return);
       continue;
     }
     break;
   }
 
   XUngrabPointer(XtDisplay(aw.shell),CurrentTime);
-  XtDispatchEvent(&e);
+  cache_cursor=(Cursor)-1;
+
+  XtDispatchEvent(event_return);
   freeze = dragging = False;
   
   /* Find out if the drag was released on the root window (child = None) */
-  XTranslateCoordinates(XtDisplay(w),e.xbutton.window,e.xbutton.root,
-			e.xbutton.x,e.xbutton.y,&x,&y,&child);
+  XTranslateCoordinates(XtDisplay(w),
+	event_return->xbutton.window,
+	event_return->xbutton.root,
+	event_return->xbutton.x,
+	event_return->xbutton.y,
+	&x,&y,&child);
   if (child == None)
     *on_root_return = True;
   else
     *on_root_return = False;
 
-  return XtWindowToWidget(XtDisplay(w),e.xbutton.window);
+  return XtWindowToWidget(XtDisplay(w),event_return->xbutton.window);
 }
 
 /*---------------------------------------------------------------------------
@@ -386,8 +409,8 @@ void filePopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
   int x, y, x_win, y_win;
   unsigned int mask;
 
-  i = findWidget(w, &fw);
-  if (!fw) {
+  i = findWidget(w, &fw, event);
+  if (i==-1 || !fw) {
 /*
     error("Internal error:", "widget not found in filePopup");
 */
@@ -427,8 +450,8 @@ void dirPopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
   int x, y, x_win, y_win;
   unsigned int mask;
 
-  i = findWidget(w, &fw);
-  if (!fw) {
+  i = findWidget(w, &fw, event);
+  if (i==-1 || !fw) {
 /*
     error("Internal error:", "widget not found in dirPopup");
 */
@@ -441,12 +464,14 @@ void dirPopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
   if (!strcmp(fw->files[i]->name, ".") ||
       !strcmp(fw->files[i]->name, "..")) {
     grayOut(dir_popup_items[2]);
-    grayOut(dir_popup_items[3]);
-    grayOut(dir_popup_items[6]);
+    grayOut(dir_popup_items[4]);
+    grayOut(dir_popup_items[5]);
+    grayOut(dir_popup_items[8]);
   } else {
     fillIn(dir_popup_items[2]);
-    fillIn(dir_popup_items[3]);
-    fillIn(dir_popup_items[6]);
+    fillIn(dir_popup_items[4]);
+    fillIn(dir_popup_items[5]);
+    fillIn(dir_popup_items[8]);
   }
 
   dpy = XtDisplay(aw.shell);
@@ -465,20 +490,24 @@ void dirPopup(Widget w, XEvent *event, String *params, Cardinal *num_params)
 void fileToggle(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
   int i;
+  
   FileWindowRec *fw;
   Pixel pix;
+  int   onewid;
 
-  i = findWidget(w, &fw);
-  if (!fw) {
+  i = findWidget(w, &fw, event);
+  if (i==-1 || !fw) {
 /*
     error("Internal error:", "widget not found in fileToggle");
 */
     return;
   }
     
-  XtVaGetValues(w, fw->files[i]->selected?XtNbackground:XtNforeground, &pix,
+  if (!(onewid=XtIsSubclass(w,fileListWidgetClass))) {
+    XtVaGetValues(w, fw->files[i]->selected?XtNbackground:XtNforeground, &pix,
 		NULL);
-  XtVaSetValues(w, XtNborder, (XtArgVal) pix, NULL);
+    XtVaSetValues(w, XtNborder, (XtArgVal) pix, NULL);
+  }
 
   if (fw->files[i]->selected) {
     fw->files[i]->selected = False;
@@ -490,7 +519,11 @@ void fileToggle(Widget w, XEvent *event, String *params, Cardinal *num_params)
     fw->n_selections++;
     fw->n_bytes_selected += fw->files[i]->stats.st_size;
   }
+  if (onewid) FileListRefreshItem((FileListWidget)w,i);
   updateStatus(fw);
+#ifdef ENHANCE_SELECTION
+  FmOwnSelection(fw,CurrentTime);
+#endif
 }
 
 /*---------------------------------------------------------------------------*/
@@ -498,11 +531,12 @@ void fileToggle(Widget w, XEvent *event, String *params, Cardinal *num_params)
 void fileSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
   int i, j;
+  int onewid=XtIsSubclass(w,fileListWidgetClass);
   FileWindowRec *fw;
   Pixel back, fore;
 
-  j = findWidget(w, &fw);
-  if (!fw) {
+  j = findWidget(w, &fw, event);
+  if (j==-1 || !fw) {
 /*
     error("Internal error:", "widget not found in fileSelect");
 */
@@ -511,20 +545,28 @@ void fileSelect(Widget w, XEvent *event, String *params, Cardinal *num_params)
   
   for (i=0; i<fw->n_files; i++)
     if (fw->files[i]->selected) {
+     fw->files[i]->selected = False;
+     if(!onewid) {
       XtCallActionProc(fw->files[i]->icon.toggle, "unset", event, NULL, 0);
       XtVaGetValues(fw->files[i]->icon.toggle, XtNbackground, &back, NULL);
       XtVaSetValues(fw->files[i]->icon.toggle, XtNborder, (XtArgVal) back,
 		    NULL);
-      fw->files[i]->selected = False;
+     } else FileListRefreshItem((FileListWidget)w,i);
     }
 
-  XtVaGetValues(w, XtNforeground, &fore, NULL);
-  XtVaSetValues(w, XtNborder, (XtArgVal) fore, NULL);
+  if (!onewid) {
+    XtVaGetValues(w, XtNforeground, &fore, NULL);
+    XtVaSetValues(w, XtNborder, (XtArgVal) fore, NULL);
+  }
 
   fw->files[j]->selected = True;
   fw->n_selections = 1;
   fw->n_bytes_selected = fw->files[j]->stats.st_size;
   updateStatus(fw);
+  if (onewid) FileListRefreshItem((FileListWidget)w,j);
+#ifdef ENHANCE_SELECTION
+  FmOwnSelection(fw,CurrentTime);
+#endif
 }
 
 /*---------------------------------------------------------------------------*/
@@ -534,11 +576,12 @@ void fileHighlight(Widget w, XEvent *event, String *params,
 {
   int i;
   FileWindowRec *fw;
+  Cursor	c;
 
   if (!dragging)
     return;
 
-  i = findWidget(w,&fw);
+  i = findWidget(w,&fw, event);
   if (!fw) {
 /*
     error("Internal error:","Widget not found in fileHighlight");
@@ -547,11 +590,16 @@ void fileHighlight(Widget w, XEvent *event, String *params,
   }
 
   if ((i == -1 && !permission(&fw->stats,W_OK)) || 
-      (i != -1 && S_ISDIR(fw->files[i]->stats.st_mode) &&
-      !permission(&fw->files[i]->stats, P_WRITE)))
+      (i != -1 && S_ISDIR(fw->files[i]->stats.st_mode) && 
+      !permission(&fw->files[i]->stats, W_OK))) c=curs[NOENTRY_CUR];
+  else					        c=drag_cursor;
+
+  if (c!=cache_cursor) {
     XChangeActivePointerGrab(XtDisplay(w),
 		       EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
-		       curs[NOENTRY_CUR], CurrentTime);
+		       c, CurrentTime);
+    cache_cursor=c;
+  }
 
   if ( i != -1 && (fw != move_info.fw || !fw->files[i]->selected))
     XtCallActionProc(w, "highlight", event, NULL, 0);
@@ -559,6 +607,38 @@ void fileHighlight(Widget w, XEvent *event, String *params,
 
 /*---------------------------------------------------------------------------*/
 
+void trackCursor(Widget w, XEvent *event, String *params,
+		 Cardinal *num_params)
+{
+  FileWindowRec *fw;
+  Cursor	c;
+
+  if (!dragging) return;
+
+  c=drag_cursor;
+
+  if (event->type!=LeaveNotify) {
+  fw=file_windows;
+  while(fw) {
+    if (w==fw->viewport || w==fw->icon_box ||
+        w==fw->label || XtParent(w)==fw->viewport) {
+          if (!permission(&fw->stats,W_OK)) c=curs[NOENTRY_CUR];
+	  break;
+    }
+    fw=fw->next;
+  }
+  }
+  if (c!=cache_cursor) {
+    XChangeActivePointerGrab(XtDisplay(w),
+		       EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
+		       c, CurrentTime);
+    cache_cursor=c;
+  }
+  
+}
+
+/*---------------------------------------------------------------------------*/
+
 void fileMaybeHighlight(Widget w, XEvent *event, String *params, 
 			Cardinal *num_params)
 {
@@ -568,7 +648,7 @@ void fileMaybeHighlight(Widget w, XEvent *event, String *params,
   if (!dragging)
     return;
 
-  i = findWidget(w,&fw);
+  i = findWidget(w,&fw, event);
   if (!fw) {
 /*
     error("Internal error:","Widget not found in fileMaybeHighlight");
@@ -577,8 +657,15 @@ void fileMaybeHighlight(Widget w, XEvent *event, String *params,
   }
 
   if (i != -1 && (fw != move_info.fw || !fw->files[i]->selected) &&
-      fw->files[i]->type && *fw->files[i]->type->drop_action)
+      fw->files[i]->type && *fw->files[i]->type->drop_action) {
+    if (cache_cursor!=drag_cursor) {
+      XChangeActivePointerGrab(XtDisplay(w),
+		       EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
+		       drag_cursor, CurrentTime);
+      cache_cursor=drag_cursor;
+    }
     XtCallActionProc(w, "highlight", event, NULL, 0);
+  }
 }
 
 /*---------------------------------------------------------------------------*/
@@ -586,25 +673,27 @@ void fileMaybeHighlight(Widget w, XEvent *event, String *params,
 void resetCursor(Widget w, XEvent *event, String *params,Cardinal *num_params)
 {
   int i;
+  Cursor c;
   FileWindowRec *fw;
-
-  i = findWidget(w,&fw);
+  i = findWidget(w,&fw, event);
   if (!fw) {
-/*
-    error("Internal error:","Widget not found in resetCursor");
-*/
     return;
   }
 
-  if (i != -1)
-    XtCallActionProc(w, "unhighlight", event, NULL, 0);
+  if (i != -1) 
+      XtCallActionProc(w, "unhighlight", event, NULL, 0);
 
   if (!dragging)
     return;
 
-  XChangeActivePointerGrab(XtDisplay(w),
+  c=((!permission(&fw->stats,W_OK))?curs[NOENTRY_CUR]:drag_cursor);
+
+  if (c!=cache_cursor) {
+    XChangeActivePointerGrab(XtDisplay(w),
 		      EnterWindowMask | LeaveWindowMask | ButtonReleaseMask, 
-		      drag_cursor, CurrentTime);
+		      c,CurrentTime);
+    cache_cursor=c;
+  }
 }
 
 
@@ -614,9 +703,13 @@ void fileRefresh(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
   FileWindowRec *fw;
   
-  findWidget(w, &fw);
-  if (fw)
+  findWidget(w, &fw, event);
+  if (fw) 
+#ifdef ENHANCE_SCROLL
+    updateFileDisplay(fw,True);
+#else
     updateFileDisplay(fw);
+#endif
 }
 
 /*---------------------------------------------------------------------------*/
@@ -626,17 +719,27 @@ void fileOpenDir(Widget w, XEvent *event, String *params,Cardinal *num_params)
   FileWindowRec *fw;
   int i;
   char path[MAXPATHLEN];
+#ifdef ENHANCE_SCROLL
+  Boolean keep_position;
+#endif
 
-  i = findWidget(w, &fw);
-  if (!fw)
+  i = findWidget(w, &fw, event);
+  if (i==-1 || !fw)
     return;
   if (chdir(fw->directory) || chdir(fw->files[i]->name))
     sysError("Can't open folder:");
   else if (!getwd(path))
     sysError("System error:");
   else {
+#ifdef ENHANCE_SCROLL
+    if (!(keep_position=(!strcmp(fw->directory,path))))
+#endif
     strcpy(fw->directory, path);
+#ifdef ENHANCE_SCROLL
+    updateFileDisplay(fw,keep_position);
+#else
     updateFileDisplay(fw);
+#endif
   }
 }
 
@@ -645,6 +748,7 @@ void fileOpenDir(Widget w, XEvent *event, String *params,Cardinal *num_params)
 void fileBeginDrag(Widget w, XEvent *event, String *params, 
 		   Cardinal *num_params)
 {
+  XEvent drag_end_event;
   int i, button;
   FileWindowRec *fw;
   Boolean root, move = True;
@@ -658,7 +762,7 @@ void fileBeginDrag(Widget w, XEvent *event, String *params,
   if (!strcmp(params[1],"copy"))
     move = False;
 
-  i =  findWidget(w, &fw);
+  i =  findWidget(w, &fw, event);
   if (!fw) {
 /*
     error("Internal error:","widget not found in fileBeginDrag");
@@ -672,7 +776,7 @@ void fileBeginDrag(Widget w, XEvent *event, String *params,
   } else
     return;
   
-  if (!(w = drag(w,button,fw,&root)))
+  if (!(w = drag(w,button,fw,&root,&drag_end_event)))
     return;
 
   /* Open directories if dragged onto the root window */
@@ -681,7 +785,7 @@ void fileBeginDrag(Widget w, XEvent *event, String *params,
     return;
   }
 
-  i = findWidget(w,&fw);
+  i = findWidget(w,&fw, &drag_end_event);
 
   /* First see whether the target is the application window */
   if (!fw) {
@@ -735,7 +839,7 @@ void fileExecFile(Widget w, XEvent *event, String *params,
   char **argv;
   FileWindowRec *fw;
 
-  i = findWidget(w, &fw);
+  i = findWidget(w, &fw, event);
   if (i == -1 || !fw) {
 /*
     error("Internal error:","widget not found in fileExecFile");
@@ -765,7 +869,7 @@ void fileExecAction(Widget w, XEvent *event, String *params,
   FileWindowRec *fw;
   char **argv;
 
-  i = findWidget(w, &fw);
+  i = findWidget(w, &fw, event);
   if (i == -1 || !fw) {
 /*
     error("Internal error:","widget not found in fileExecAction");
@@ -869,3 +973,27 @@ void doView(char *directory, char *fname)
   else
     error("No default viewer", "");
 }
+/*---------------------------------------------------------------------------*/
+void doXterm(char *directory)
+{
+  char path[MAXPATHLEN];
+  char **argv;
+
+  if (resources.default_xterm) {
+
+    strcpy(path, resources.default_xterm);
+
+    argv = (char **) XtMalloc(4 * sizeof(char *));
+    argv[0] = user.shell;
+    argv[1] = "-c";
+    argv[2] = path;
+    argv[3] = NULL;
+
+    executeApplication(user.shell, directory, argv);
+  
+    XTFREE(argv);
+  }
+
+  else
+    error("No default xterm", "");
+}
