@@ -131,21 +131,9 @@ void infoOkCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 ---------------------------------------------------------------------------*/
 
 static ButtonRec info_buttons[] = {
-  { "ok", "Ok", infoOkCb }
+  { "cancel", "Cancel", infoOkCb }
 };
 
-/*---------------------------------------------------------------------------
-  WM_DELETE_WINDOW handler
----------------------------------------------------------------------------*/
-
-static void clmHandler(Widget w, XtPointer cld, XEvent *ev, Boolean *ctd)
-{
-  XClientMessageEvent *cme = (XClientMessageEvent *)ev;
-  if (ev->type != ClientMessage || cme->message_type != wm_protocols)
-    return;
-  infoOkCb(w, NULL, NULL);
-}
-
 /*---------------------------------------------------------------------------
   PUBLIC FUNCTIONS
 ---------------------------------------------------------------------------*/
@@ -197,13 +185,11 @@ void createInfoPopup()
 
 #ifdef ENHANCE_POP_ACCEL
   XtInstallAllAccelerators(form,form);
+  XtInstallAllAccelerators(info.shell,form);
 #endif
 
   XtRealizeWidget(info.shell);
-  XSetWMProtocols(XtDisplay(info.shell), XtWindow(info.shell),
-		  &wm_delete_window, 1);
-  XtAddEventHandler(info.shell, (EventMask)0L, True,
-		    (XtEventHandler)clmHandler, (XtPointer)NULL);
+  setWMProps(info.shell);
 }
 
 /*---------------------------------------------------------------------------*/
