@@ -477,11 +477,11 @@ void noWarnings(String msg)
 static char app_ver0[100], *app_ver;
 #endif
 
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
   char *s;
 #ifdef ENHANCE_TRANSLATIONS
-  char *p;
+  char *p, *xfmversion = XFMVERSION;
 #endif
 
   progname = argv[0];
@@ -512,6 +512,10 @@ void main(int argc, char *argv[])
 			     &argc, argv, fallback_resources, shell_args,
 			     XtNumber(shell_args) );
 
+  /* First, initialize the atoms */
+  /* initialise the communications module */
+  initComms();
+
   /* make sure we can close-on-exec the display connection */
   if (fcntl(ConnectionNumber(XtDisplay(aw.shell)), F_SETFD, 1) == -1)
     abortXfm("Couldn't mark display connection as close-on-exec");
@@ -528,10 +532,17 @@ void main(int argc, char *argv[])
 
   /* -version: print version number and exit: */
   if (resources.version) {
-    printf("xfm version %s\n", XFMVERSION);
+    printf("xfm version %s.%s\n", xfmversion,XFMMINORVERSION);
     exit(0);
   }
 
+#ifdef ENHANCE_LOG
+  /* we need the 'tick' bitmap already here */
+  readBitmaps();
+  (void)FmCreateLog(aw.shell,resources.button_font);
+#endif
+
+
 #ifdef ENHANCE_USERINFO
   getwd(user.cwd);
   if (!(s = getenv("HOME"))) s=user.cwd;
@@ -545,15 +556,6 @@ void main(int argc, char *argv[])
   }
 #endif
 
-  /* initialise the communications module */
-  initComms();
-
-#ifdef ENHANCE_LOG
-  /* we need the 'tick' bitmap already here */
-  readBitmaps();
-  (void)FmCreateLog(aw.shell,resources.button_font);
-#endif
-
   /* default is to launch both application and file manager */
   if (!resources.appmgr && !resources.filemgr)
     resources.appmgr = resources.filemgr = True;
@@ -603,14 +605,14 @@ void main(int argc, char *argv[])
   app_ver = app_ver0;
   if (*app_ver==(char)'"') app_ver++;
   if (p=strrchr(app_ver,'"')) *p=0;
-  if (strcmp(app_ver,XFMVERSION)) {
+  if (strcmp(app_ver,xfmversion)) {
     if (*app_ver ==(char)0)
 	error("Sorry: Appl. Defaults Not Found",
 	      "check XFILESEARCHPATH and friends");
     else {
 	p=XtMalloc(128);
 	sprintf(p,"found: %s, needed: %s",
-	        app_ver,XFMVERSION);
+	        app_ver,xfmversion);
 	error("Appl. Defaults: wrong version number",p);
         XTFREE(p);
     }
@@ -649,13 +651,21 @@ void main(int argc, char *argv[])
     readApplicationData(resources.app_file);
     createApplicationWindow();
     createApplicationDisplay();
-    XtRealizeWidget(aw.shell);
-    setApplicationWindowName();
-    XSetWMProtocols(XtDisplay(aw.shell), XtWindow(aw.shell),
-		    &wm_delete_window, 1);
-    XtAddEventHandler(aw.shell, (EventMask)0L, True,
-		      (XtEventHandler)clientMessageHandler, (XtPointer)NULL);
+  } else {
+	/* create a dummy window to store the WM_COMMAND property */
+	XtVaSetValues(aw.shell,
+		XtNmappedWhenManaged,False,
+		0);
   }
+  /* still realize and set the WM properties. the WM needs
+   * a window around and kwm is even able to restart an app
+   * without a mapped toplevel window.
+   */
+  XtRealizeWidget(aw.shell);
+  setApplicationWindowName();
+  setWMProps(aw.shell);
+  XtAddEventHandler(aw.shell, (EventMask)0L, True,
+	    (XtEventHandler)clientMessageHandler, (XtPointer)NULL);
 
 #ifdef ENHANCE_SELECTION
   /* initialize the selection ICCM */
@@ -684,6 +694,7 @@ void main(int argc, char *argv[])
 
   /* collect & process events */
   XtAppMainLoop(app_context);
+  return 0;
 }
 
 /*---------------------------------------------------------------------------*/
