@@ -9,6 +9,9 @@
   modified 1-29-95 by rodgers@lvs-emh.lvs.loral.com (Kevin M. Rodgers)
   to add filtering of icon/text directory displays by a filename filter.
 
+  modified 7-1997 by strauman@sun6hft.ee.tu-berlin.de to add
+  different enhancements (see README-NEW).
+
 ---------------------------------------------------------------------------*/
 
 #include <stdio.h>
@@ -21,21 +24,33 @@
 #include <X11/Xaw/Dialog.h>
 #include <X11/Xaw/Toggle.h>
 
+#include "FileList.h"
+
 #include "Fm.h"
 
+#ifdef ENHANCE_HISTORY
+#include "Am.h" 	/* need to know aw.shell */
+#include "FmHistory.h"
+#endif
+
+#ifdef ENHANCE_SELECTION
+#include "FmSelection.h"
+#endif
+
 /*---------------------------------------------------------------------------
   STATIC DATA
 ---------------------------------------------------------------------------*/
 
 typedef struct {
   FileWindowRec *fw;
-  Widget mkdir, createFile, goTo, move, copy, link, select, filter;
+  Widget mkdir, createFile, goTo, rename, move, copy, link, select, filter;
   char s[MAXPATHLEN], t[MAXPATHLEN];
   char select_s[MAXPATHLEN];
   char filter_s[MAXPATHLEN];
   char mkdir_s[MAXPATHLEN];
   char createFile_s[MAXPATHLEN];
   char goTo_s[MAXPATHLEN];
+  char rename_s[MAXPATHLEN];
   char move_s[MAXPATHLEN];
   char copy_s[MAXPATHLEN];
   char link_s[MAXPATHLEN];
@@ -121,18 +136,44 @@ static void createFileCancelCb(Widget w, FileWindowRec *fw,
 static void goToOkCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 {
   char path[MAXPATHLEN];
+#ifdef ENHANCE_SCROLL
+  Boolean keep_position=True;
+#endif
 
   XtPopdown(popups.goTo);
   fnexpand(popups.goTo_s);
+#ifdef ENHANCE_BUGFIX
+  /* if it's impossible to chdir(popups.fw->directory)
+   * it still might be possible to chdir(popups.goTo_s) !
+   * eg. if the actual directory was deleted, an we want to
+   * change to another one.
+   */
+  if (chdir(popups.goTo_s)) {
+    char s[0xff];
+    sprintf(s, "Can't open folder %s:", popups.goTo_s);
+    sysError(s);
+	if (chdir(popups.fw->directory)) {
+    	  sprintf(s, "Can't open folder %s:", popups.fw->directory);
+    	  sysError(s);
+	}
+#else
   if (chdir(popups.fw->directory) || chdir(popups.goTo_s)) {
     char s[0xff];
     sprintf(s, "Can't open folder %s:", popups.goTo_s);
     sysError(s);
+#endif
   } else if (!getwd(path))
     sysError("System error:");
   else {
+#ifdef ENHANCE_SCROLL
+    if (!(keep_position= (!strcmp(popups.fw->directory,path))))
+#endif
     strcpy(popups.fw->directory, path);
+#ifdef ENHANCE_SCROLL
+    updateFileDisplay(popups.fw,keep_position);
+#else
     updateFileDisplay(popups.fw);
+#endif
   }
   freeze = False;
 }
@@ -147,6 +188,75 @@ static void goToCancelCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 
 /*---------------------------------------------------------------------------*/
 
+static void renameOkCb(Widget w, FileWindowRec *fw, XtPointer call_data)
+{
+  struct stat stats;
+  int i;
+  char *from = NULL, to[MAXPATHLEN], todir[MAXPATHLEN];
+
+  XtPopdown(popups.rename);
+  strcpy(to, popups.rename_s);
+  fnexpand(to);
+
+  if (chdir(popups.fw->directory))
+
+    sysError("System error:");
+
+  else {
+
+    struct stat stats1;
+
+    for (i = 0; i < popups.fw->n_files; i++)
+      if (popups.fw->files[i]->selected) {
+	from = popups.fw->files[i]->name;
+	break;
+      }
+
+    if (!strcmp(from, ".") || !strcmp(from, "..")) {
+      error("Cannot rename . or ..", "");
+      goto out;
+    } else if (!lstat(to, &stats) && !lstat(from, &stats1) &&
+	       stats.st_ino == stats1.st_ino) {
+      error("Rename:", "Source and destination are identical");
+      goto out;
+    }
+
+    if (exists(to) && resources.confirm_overwrite) {
+      char s[0xff];
+      sprintf(s, "Rename: file %s already exists", to);
+      if (!confirm(s, "Overwrite?", ""))
+	goto out;
+    }
+
+    if (rmove(from, to)) {
+      char s[0xff];
+      sprintf(s, "Error renaming %s:", from);
+      sysError(s);
+    } else {
+      dir_prefix(todir, to);
+      if ((*todir?chdir(todir):0) || !getwd(todir))
+	sysError("System error:");
+      markForUpdate(popups.fw->directory); markForUpdate(todir);
+      intUpdate();
+    }
+
+  }
+
+ out:
+  freeze = False;
+}
+
+/*---------------------------------------------------------------------------*/
+
+static void renameCancelCb(Widget w, FileWindowRec *fw, 
+			   XtPointer call_data)
+{
+  XtPopdown(popups.rename);
+  freeze = False;
+}
+
+/*---------------------------------------------------------------------------*/
+
 static void moveOkCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 {
   struct stat stats;
@@ -202,7 +312,7 @@ static void moveOkCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 	      else
 		continue;
 	  }
-	  if (rename(from,to)) {
+	  if (rmove(from,to)) {
 	    char s[0xff];
 	    sprintf(s, "Error moving %s:", from);
 	    sysError(s);
@@ -245,7 +355,7 @@ static void moveOkCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 	  goto out;
       }
 
-      if (rename(from, to)) {
+      if (rmove(from, to)) {
 	char s[0xff];
 	sprintf(s, "Error moving %s:", from);
 	sysError(s);
@@ -557,28 +667,39 @@ static void selectReplaceCb(Widget w, FileWindowRec *fw,
 {
   int i;
   Pixel pix;
+  Boolean new_val;
+  int onewid=XtIsSubclass(popups.fw->icon_box,fileListWidgetClass);
 
   XtPopdown(popups.select);
   popups.fw->n_selections = 0;
   popups.fw->n_bytes_selected = 0;
   for (i=0; i<popups.fw->n_files; i++) {
-    if (popups.fw->files[i]->icon.toggle) {
+    if (onewid || popups.fw->files[i]->icon.toggle) {
       if (fnmatchnodot(popups.select_s, popups.fw->files[i]->name)) {
-	popups.fw->files[i]->selected = True;
+	new_val = True;
 	popups.fw->n_selections++;
 	popups.fw->n_bytes_selected += popups.fw->files[i]->stats.st_size;
       }
       else
-	popups.fw->files[i]->selected = False;
-      XtVaGetValues(popups.fw->files[i]->icon.toggle,
-		    popups.fw->files[i]->selected?XtNforeground:XtNbackground,
-		    &pix, NULL);
-      XtVaSetValues(popups.fw->files[i]->icon.toggle, XtNborder,
-		    (XtArgVal) pix, NULL);
+	new_val = False;
+      if (new_val!= popups.fw->files[i]->selected) {
+	popups.fw->files[i]->selected = new_val;
+        if (!onewid) {
+          XtVaGetValues(popups.fw->files[i]->icon.toggle,
+	  	      popups.fw->files[i]->selected?XtNforeground:XtNbackground,
+		      &pix, NULL);
+          XtVaSetValues(popups.fw->files[i]->icon.toggle, XtNborder,
+		      (XtArgVal) pix, NULL);
+        } else 
+	  FileListRefreshItem((FileListWidget)popups.fw->icon_box,i);
+     }
     }
   }
   updateStatus(popups.fw);
   freeze = False;
+#ifdef ENHANCE_SELECTION
+  FmOwnSelection(popups.fw,CurrentTime);
+#endif
 }
 
 /*---------------------------------------------------------------------------*/
@@ -587,23 +708,29 @@ static void selectAddCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 {
   int i;
   Pixel pix;
+  int onewid=XtIsSubclass(popups.fw->icon_box,fileListWidgetClass);
   
   XtPopdown(popups.select);
   for(i=0; i<popups.fw->n_files; i++)
-    if (popups.fw->files[i]->icon.toggle) {
+    if (onewid || popups.fw->files[i]->icon.toggle) {
       if (!popups.fw->files[i]->selected && 
 	  (fnmatchnodot(popups.select_s, popups.fw->files[i]->name))) {
 	popups.fw->files[i]->selected = True;
 	popups.fw->n_selections++;
 	popups.fw->n_bytes_selected += popups.fw->files[i]->stats.st_size;
-	XtVaGetValues(popups.fw->files[i]->icon.toggle, XtNforeground, &pix,
+	if (!onewid) {
+	  XtVaGetValues(popups.fw->files[i]->icon.toggle, XtNforeground, &pix,
 		      NULL);
-	XtVaSetValues(popups.fw->files[i]->icon.toggle, XtNborder,
+	  XtVaSetValues(popups.fw->files[i]->icon.toggle, XtNborder,
 		      (XtArgVal) pix, NULL);
+	} else FileListRefreshItem((FileListWidget)popups.fw->icon_box,i);
       }
     }
   updateStatus(popups.fw);
   freeze = False;
+#ifdef ENHANCE_SELECTION
+  FmOwnSelection(popups.fw,CurrentTime);
+#endif
 }
 
 /*---------------------------------------------------------------------------*/
@@ -612,23 +739,29 @@ static void selectRemoveCb(Widget w, FileWindowRec *fw, XtPointer call_data)
 {
   int i;
   Pixel pix;
+  int onewid=XtIsSubclass(popups.fw->icon_box,fileListWidgetClass);
   
   XtPopdown(popups.select);
   for(i=0; i<popups.fw->n_files; i++)
-    if (popups.fw->files[i]->icon.toggle) {
+    if (onewid || popups.fw->files[i]->icon.toggle) {
       if (popups.fw->files[i]->selected && 
 	  (fnmatch(popups.select_s, popups.fw->files[i]->name))) {
 	popups.fw->files[i]->selected = False;
 	popups.fw->n_selections--;
 	popups.fw->n_bytes_selected -= popups.fw->files[i]->stats.st_size;
-	XtVaGetValues(popups.fw->files[i]->icon.toggle, XtNbackground, &pix,
+	if (!onewid) {
+	  XtVaGetValues(popups.fw->files[i]->icon.toggle, XtNbackground, &pix,
 		      NULL);
-	XtVaSetValues(popups.fw->files[i]->icon.toggle, XtNborder,
+	  XtVaSetValues(popups.fw->files[i]->icon.toggle, XtNborder,
 		      (XtArgVal) pix, NULL);
+	} else FileListRefreshItem((FileListWidget)popups.fw->icon_box,i);
       }
     }
   updateStatus(popups.fw);
   freeze = False;
+#ifdef ENHANCE_SELECTION
+  FmOwnSelection(popups.fw,CurrentTime);
+#endif
 }
 
 /*---------------------------------------------------------------------------*/
@@ -659,7 +792,11 @@ static void filterOkCb(Widget w, FileWindowRec *fw,
   XtPopdown(popups.filter);
   popups.fw->do_filter = True;
   strcpy(popups.fw->dirFilter,popups.filter_s);
+#ifdef ENHANCE_SCROLL
+  updateFileDisplay(popups.fw,True);
+#else
   updateFileDisplay(popups.fw);
+#endif
   freeze = False;
 }
 
@@ -672,7 +809,11 @@ static void filterClearCb(Widget w, FileWindowRec *fw,
   XtPopdown(popups.filter);
   popups.fw->do_filter = False;
   popups.fw->dirFilter[0] = '\0';
+#ifdef ENHANCE_SCROLL
+  updateFileDisplay(popups.fw,True);
+#else
   updateFileDisplay(popups.fw);
+#endif
   freeze = False;
 }
 
@@ -695,6 +836,11 @@ static ButtonRec goTo_buttons[] = {
   { "cancel", "Cancel", goToCancelCb }
 };
 
+static ButtonRec rename_buttons[] = {
+  { "ok", "Ok", renameOkCb },
+  { "cancel", "Cancel", renameCancelCb }
+};
+
 static ButtonRec move_buttons[] = {
   { "ok", "Ok", moveOkCb },
   { "cancel", "Cancel", moveCancelCb }
@@ -739,6 +885,10 @@ static QuestionRec goTo_questions[] = {
   { "Go to folder:", popups.goTo_s, MAXPATHLEN, NULL }
 };
 
+static QuestionRec rename_questions[] = {
+  { "Rename to:", popups.rename_s, MAXPATHLEN, NULL }
+};
+
 static QuestionRec move_questions[] = {
   { "Move to:", popups.move_s, MAXPATHLEN, NULL }
 };
@@ -759,53 +909,102 @@ static QuestionRec filter_questions[] = {             /* KMR */
   { "Filename pattern:", popups.filter_s, MAXPATHLEN, NULL }
 };
 
+#ifdef ENHANCE_HISTORY
+/*--------------------------------------------------------------------------
+  PUBLIC VARIABLES
+--------------------------------------------------------------------------*/
+
+HistoryList path_history;
+
+#endif
+
 /*---------------------------------------------------------------------------
   PUBLIC FUNCTIONS
 ---------------------------------------------------------------------------*/
 
 void createMainPopups()
 {
+#ifdef ENHANCE_HISTORY
+  char *fixed[5];
+  int	nfixed;
+
+  /* Path History menu is initialized
+   * (need to do this first because the history feature
+   * provides an additional action that must be added to the action table
+   * so the different popups can use it :-)
+   */
+
+  nfixed=0;
+  fixed[nfixed++]=user.home;
+#ifdef ENHANCE_USERINFO
+  fixed[nfixed++]=user.cwd;
+#endif
+  fixed[nfixed++]=0;
+  path_history=FmCreateHistoryList(
+	0, /* let the menu's name be "fm_history" */
+	aw.shell,
+	fixed);
+
+  /* Note: important resources of the history menu are _not_ configured
+   *       statically but by the resource files.
+   */
+#endif
+
   /* New Folder */
   popups.mkdir = createPopupQuestions("mkdir", "New Folder", bm[DIR_BM], 
 			       mkdir_questions, XtNumber(mkdir_questions),
-			       mkdir_buttons, XtNumber(mkdir_buttons) );
+			       mkdir_buttons, XtNumber(mkdir_buttons),
+			       XtNumber(mkdir_buttons)-1);
 
   /* New File */
   popups.createFile = createPopupQuestions("create", "New File", bm[FILE_BM],
 			       createFile_questions,
 			       XtNumber(createFile_questions),
 			       createFile_buttons,
-			       XtNumber(createFile_buttons) );
+			       XtNumber(createFile_buttons),
+			       XtNumber(createFile_buttons)-1);
 
   /* Change current folder */
   popups.goTo = createPopupQuestions("goto", "Go To", bm[DIR_BM],
 			       goTo_questions, XtNumber(goTo_questions),
-			       goTo_buttons, XtNumber(goTo_buttons) );
+			       goTo_buttons, XtNumber(goTo_buttons),
+			       XtNumber(goTo_buttons)-1);
+
+  /* Rename file */
+  popups.rename = createPopupQuestions("rename", "Rename", bm[FILE_BM], 
+			       rename_questions, XtNumber(rename_questions),
+			       rename_buttons, XtNumber(rename_buttons),
+			       XtNumber(rename_buttons)-1);
 
   /* Move file */
   popups.move = createPopupQuestions("move", "Move", bm[FILES_BM], 
 			       move_questions, XtNumber(move_questions),
-			       move_buttons, XtNumber(move_buttons) );
+			       move_buttons, XtNumber(move_buttons),
+			       XtNumber(move_buttons)-1);
 
   /* Copy file */
   popups.copy = createPopupQuestions("copy", "Copy", bm[FILES_BM],
 			       copy_questions, XtNumber(copy_questions),
-			       copy_buttons, XtNumber(copy_buttons) );
+			       copy_buttons, XtNumber(copy_buttons),
+			       XtNumber(copy_buttons)-1);
 
   /* Create link */
   popups.link = createPopupQuestions("link", "Link", bm[SYMLNK_BM],
 			       link_questions, XtNumber(link_questions),
-			       link_buttons, XtNumber(link_buttons) );
+			       link_buttons, XtNumber(link_buttons),
+			       XtNumber(link_buttons)-1);
 
   /* Select */
   popups.select = createPopupQuestions("select", "Select", bm[FILES_BM],
 			       select_questions, XtNumber(select_questions),
-			       select_buttons, XtNumber(select_buttons) );
+			       select_buttons, XtNumber(select_buttons),
+			       XtNumber(select_buttons)-1);
 
   /* Filter */  /* KMR */
   popups.filter = createPopupQuestions("filter", "Filter", bm[FILES_BM],
 			       filter_questions, XtNumber(filter_questions),
-			       filter_buttons, XtNumber(filter_buttons) );
+			       filter_buttons, XtNumber(filter_buttons),
+			       XtNumber(filter_buttons)-1);
 
   /* Change Access Mode */
   createChmodPopup();
@@ -818,6 +1017,7 @@ void createMainPopups()
 
   /* Deletions */
   createConfirmPopup();
+
 }
 
 /*---------------------------------------------------------------------------*/ 
@@ -879,6 +1079,28 @@ void goToPopup(Widget w, FileWindowRec *fw, XtPointer call_data)
 
 /*---------------------------------------------------------------------------*/
 
+void renamePopup(Widget w, FileWindowRec *fw, XtPointer call_data)
+{
+  int i;
+
+  if (fw == NULL) fw = popup_fw;
+
+  if (!fw->n_selections) return;
+
+  popups.fw = fw;
+  for (i = 0; i < popups.fw->n_files; i++)
+    if (popups.fw->files[i]->selected) {
+      strcpy(popups.rename_s, popups.fw->files[i]->name);
+      break;
+    }
+  XtVaSetValues(rename_questions[0].widget, XtNstring, 
+		(XtArgVal) popups.rename_s, NULL);
+  freeze = True;
+  popupByCursor(popups.rename, XtGrabExclusive);
+}
+
+/*---------------------------------------------------------------------------*/
+
 void movePopup(Widget w, FileWindowRec *fw, XtPointer call_data)
 {
   if (fw == NULL) fw = popup_fw;
