@@ -285,15 +285,6 @@ Widget *createButtons(ButtonList buttons, Cardinal n_buttons, Widget parent,
 /* the bitmap, and a set of buttons underneath all this.                     */
 /*****************************************************************************/
 
-static void clmHandler(Widget w, XtPointer cld, XEvent *ev, Boolean *ctd)
-{
-  XtCallbackProc cb = (XtCallbackProc)cld;
-  XClientMessageEvent *cme = (XClientMessageEvent *)ev;
-  if (ev->type != ClientMessage || cme->message_type != wm_protocols)
-    return;
-  if (cb) cb(w, NULL, NULL);
-}
-
 Widget createPopupQuestions(String name, String title, Pixmap bitmap,
 			    QuestionList questions, Cardinal n_questions,
 			    ButtonList buttons, Cardinal n_buttons,
@@ -379,6 +370,7 @@ Widget createPopupQuestions(String name, String title, Pixmap bitmap,
     	XtInstallAllAccelerators(questions[i].widget,form_widget);
     }
     XtInstallAllAccelerators(form_widget,form_widget);
+    XtInstallAllAccelerators(shell,form_widget);
 #ifndef ENHANCE_TRANSLATIONS
     for (i=0;i<n_buttons;i++) {
 	/* This allows the user to trigger the buttons
@@ -394,7 +386,7 @@ Widget createPopupQuestions(String name, String title, Pixmap bitmap,
 	  XtNaccelerators,
 	  (flag==1?
 	    "<Key>Return:set()notify()unset()\n <Key>Linefeed:set()notify()unset()\n":
-	    "<Key>Escape:set()notify()unset()\n"),
+	    "<ClientMessage>WM_PROTOCOLS:set()notify()unset()\n<Key>Escape:set()notify()unset()\n"),
 	  NULL);
       }
     }
@@ -403,15 +395,7 @@ Widget createPopupQuestions(String name, String title, Pixmap bitmap,
   }
 
   XtRealizeWidget(shell);
-  XSetWMProtocols(XtDisplay(shell), XtWindow(shell),
-		  &wm_delete_window, 1);
-  if (buttons && 0 <= dflt_button && dflt_button < n_buttons)
-    XtAddEventHandler(shell, (EventMask)0L, True,
-		      (XtEventHandler)clmHandler,
-		      (XtPointer)buttons[dflt_button].callback);
-  else
-    XtAddEventHandler(shell, (EventMask)0L, True,
-		      (XtEventHandler)clmHandler, (XtPointer)NULL);
+  setWMProps(shell);
 
   return shell;
 }
@@ -727,6 +711,7 @@ static Widget createPopupText(String name, String title, Pixmap bitmap,
 
 #ifdef ENHANCE_POP_ACCEL
     XtInstallAllAccelerators(form_widget,form_widget);
+    XtInstallAllAccelerators(shell,form_widget);
 #ifndef ENHANCE_TRANSLATIONS
     for (i=0;i<n_buttons;i++) {
 	/* This allows the user to trigger the buttons
@@ -751,15 +736,7 @@ static Widget createPopupText(String name, String title, Pixmap bitmap,
   }
 
   XtRealizeWidget(shell);
-  XSetWMProtocols(XtDisplay(shell), XtWindow(shell),
-		  &wm_delete_window, 1);
-  if (buttons && 0 <= dflt_button && dflt_button < n_buttons)
-    XtAddEventHandler(shell, (EventMask)0L, True,
-		      (XtEventHandler)clmHandler,
-		      (XtPointer)buttons[dflt_button].callback);
-  else
-    XtAddEventHandler(shell, (EventMask)0L, True,
-		      (XtEventHandler)clmHandler, (XtPointer)NULL);
+  setWMProps(shell);
 
   return shell;
 }
@@ -781,7 +758,7 @@ static ButtonRec about_buttons[] = {
   { "ok", "Ok", (FmCallbackProc *) aboutOkCb },
 };
 
-static char ver_str[100];
+static char ver_str[250];
 static String about_text[] = {
   ver_str,
   "Written by Simon Marlow, Albert Graef, Till Straumann, et al",
@@ -793,7 +770,7 @@ static String about_text[] = {
 void aboutPopup(void)
 {
   if (about == None) {
-    sprintf(ver_str, "This is xfm version %s", XFMVERSION);
+    sprintf(ver_str, "This is xfm version %s.%s", XFMVERSION,XFMMINORVERSION);
     logo_bm = readIcon("XfmLogo.xpm", 0, 0);
     about = createPopupText("about", "About xfm", logo_bm,
 			    about_text, XtNumber(about_text), about_buttons,
@@ -803,3 +780,37 @@ void aboutPopup(void)
   if (!XtIsManaged(about))
     popupByCursor(about, XtGrabNone);
 }
+
+/* set the neccessary WM properties */
+void setWMProps(Widget shell)
+{
+if (!XtIsShell(shell) || !XtIsRealized(shell)) {
+	String msg=XtMalloc(100);
+	sprintf(msg,"setWMProps(%s): need a realized shell",XtName(shell));
+	abortXfm(msg);
+	XtFree(msg); /* not really needed */
+}
+if (!XtIsApplicationShell(shell)) {
+	char *empty="";
+	XSetWMProtocols(XtDisplay(shell), XtWindow(shell), &wm_delete_window, 1);
+	/* clear the WM_COMMAND property; work around some kwm problems */
+	XSetCommand(XtDisplay(shell),XtWindow(shell),&empty,1);
+} else {
+	Atom atoms[2];
+	/* set the SAVE_YOURSELF protocol as well */
+	atoms[0]=wm_delete_window; atoms[1]=wm_save_yourself;
+	XSetWMProtocols(XtDisplay(shell), XtWindow(shell), atoms, XtNumber(atoms));
+	/* is the KWM_SAVE_YOURSELF atom known ? */
+	if (None != kwm_save_yourself) {
+		long val=0x1;
+		/* seems that kwm is running; if we set KWM_SAVE_YOURSELF to 0x1, 
+		 * our session will also be restored if we ran `xfm -filemgr' :-)
+		 */
+		XChangeProperty(XtDisplay(shell),XtWindow(shell),
+						kwm_save_yourself, kwm_save_yourself,
+						32,
+						PropModeReplace,
+						(unsigned char *)&val,1);
+	}
+}
+}
