@@ -532,7 +532,7 @@ char buf[128], *chpt=0;
 struct group *grp;
 
 for (i=0;i<FL.n_files;i++) {
-   if (chpt && gid==(tmpgid=FL.files[i]->stats.st_gid)) continue;
+   if (gid==(tmpgid=FL.files[i]->stats.st_gid) && chpt) continue;
    grp=getgrgid(gid=tmpgid);
    if (!grp) {
 	sprintf(buf,"%lu",(unsigned long)tmpgid);
