@@ -38,6 +38,7 @@ static int     hdrbufsiz = 0;	/* Size of header buffer. */
 static struct stat stbuf;	/* Stat buffer. */
 static struct stat lstbuf;	/* Lstat buffer. */
 static int bytes;		/* Bytes read from the file. */
+static char   *namep;           /* pointer to filename */
 static int     mmatch();
 static char   *builtin_test();
 static char   *builtin_result;
@@ -49,6 +50,8 @@ char *name;
     FILE *fh;
     int hsiz;
 
+    count = 0; /* Added by O. Mai to allow reparsing of magic file */
+
     if(!hdrbuf)
     {
 	hdrbuf = (char *) malloc(maxhdr + 1);
@@ -224,7 +227,7 @@ char *name;
 		cptr++;
 		break;
 	    case '^':
-		mtypes[count].flags |= M_OR;
+		mtypes[count].flags |= M_CLEAR;
 		cptr++;
 		break;
 	    case 'x':
@@ -262,6 +265,16 @@ char *buf;
     int i;
     int fd;
 
+    namep = name;
+
+    /* added hdrbuf initialization in case magic_get_type() is invoked
+       before magic_parse_file() - AG */
+    if(!hdrbuf)
+    {
+       hdrbuf = (char *) malloc(maxhdr + 1);
+       hdrbufsiz = maxhdr;
+    }
+
     if(lstat(name, &lstbuf) < 0)
     {
 	stbuf.st_mode = 0;
@@ -469,8 +482,9 @@ char *buf;
 	}
 	else
 	    return 0;
-    case M_OR:
-	if((v & mtypes[i].value.number) != 0)
+    case M_CLEAR:
+        /* AG fix: this must be "any clear" instead of "or" */
+        if((~v & mtypes[i].value.number) != 0)
 	{
 	    sprintf(buf, mtypes[i].message, v);
 	    return 1;
