@@ -24,7 +24,7 @@
 #define M_LT		(0x1<<5)
 #define M_GT		(0x2<<5)
 #define M_SET		(0x3<<5)
-#define M_OR		(0x4<<5)
+#define M_CLEAR		(0x4<<5)
 #define M_ANY		(0x5<<5)
 #define M_OP		(0x7<<5)	/* Mask for operations. */
 
